################################################
## Structuration des donnees pour sunburst
################################################


#########################
## Appel aux librairies
#########################


## initialisation de t_data
t_data <- data_file

library(sqldf)


#############################
## Structuration avec boucle
#############################



## Preparation de la table si un sens de sequence est demande ---------------------------------------
if(sens_sequence != 'brut'){
## Determiner le hit de la page a etudier  
query<- paste0(" SELECT MIN(hitNumber) as min_hit_page, session_id as session_id_pour_min
                 FROM (SELECT *
                        FROM t_data
                        WHERE page ='", page_a_etudier ,"'
                      )
                 GROUP BY session_id
                    ")
t_data_page <- sqldf(query)

## Filtrer les hits à prendre en compte à partir du hit determiné par session ---------------------
if(sens_sequence == 'normal'){
  sens_filtre <- '>'
} else if(sens_sequence == 'inverse'){
  sens_filtre <- '<'
}
query<- paste0(" SELECT *  
                 FROM t_data a
                 LEFT JOIN t_data_page b ON a.session_id=b.session_id_pour_min 
                 WHERE a.hitNumber ", sens_filtre, "= b.min_hit_page
                 ORDER BY session_id, hitNumber
                    ")
t_data <- sqldf(query)
}                     

## Inverser le hitNumber pour le sens de sequence inverse (pour commencer par page étudiée) -----------------------------
if(sens_sequence == 'inverse'){
query<- " SELECT a.session_id as session_id, (b.max_hitNumber - a.hitNumber + 1) as hitNumber, a.page as page
                 FROM t_data a
                 LEFT JOIN (
                   SELECT MAX(hitNumber) as max_hitNumber, session_id as session_id_pour_max
                   FROM t_data
                   GROUP BY session_id
                   ) as b ON a.session_id=b.session_id_pour_max
                 ORDER BY session_id, hitNumber "
t_data <- sqldf(query)
} 


## Recuperer le nombre de pages maximum. Pour la boucle ------------------------------------------------------------
nb_pages_max <- sqldf("
                      SELECT max(pagesParSession) as pagesMax
                      FROM (
                        SELECT session_id, count(hitNumber) as pagesParSession
                        FROM t_data
                        GROUP BY session_id
                        )
                      ")

## Niveaux de pages a avoir dans la structure finale ---------------------------------------------------------------
if(nb_niveaux_pages == 0){
  nb_pages_max_pour_boucle <- nb_pages_max$pagesMax + 1
}else{
  nb_pages_max_pour_boucle <- nb_niveaux_pages
}


##
## Ajuster les hits number pour avoir un ordre simplifie (1,2,3, etc)
##

##
## Initialisation de la colonne hitNumber_adjusted (pour hitNumber=1)
##

## calcul du minimum pour hitNumber = 1 ---------------------------------------------------------------------
t_hitNumber_adjusted <- sqldf(
  "SELECT t_data.session_id, t_data.hitNumber, b.min as min_hitNumber
  FROM t_data
  LEFT JOIN (SELECT session_id, MIN(hitNumber) as min FROM t_data GROUP BY session_id) as b ON t_data.session_id = b.session_id
  "
)

## calcul du hitNumber_adjusted -----------------------------------------------------------------------------
t_hitNumber_adjusted <- sqldf(
  "SELECT *,
  CASE WHEN hitNumber = min_hitNumber 
  THEN 1 
  ELSE hitNumber 
  END as hitNumber_adjusted
  FROM t_hitNumber_adjusted
  "
)

##
## Iteration pour chaque niveau (a partir de hitNumber=2)
##


## DEBUT boucle
for (i in 2:nb_pages_max_pour_boucle){

## ajout de la colonne d'aide pour le calcul du minimum pour hitNumber=2 -----------------------------------------
query <- paste0('
        SELECT session_id, hitNumber, hitNumber_adjusted, 
        CASE WHEN hitNumber_adjusted <= ', i-1, '
        THEN NULL ELSE hitNumber_adjusted END as hitNumber_adjusted_temp
        FROM t_hitNumber_adjusted
                ')
t_hitNumber_adjusted <- sqldf(query)

## calcul du minimum pour hitNumber=2 ----------------------------------------------------------------------------
query <- paste0('
              SELECT a.*, b.min as min_hitNumber
              FROM t_hitNumber_adjusted as a
              LEFT JOIN ( SELECT session_id, MIN(hitNumber_adjusted_temp) as min FROM t_hitNumber_adjusted GROUP BY session_id ) as b ON a.session_id = b.session_id
              ')
t_hitNumber_adjusted <- sqldf(query)

## calcul du hitNumber_adjusted -------------------------------------------------------------------------------------
query <- paste0('
                SELECT session_id, hitNumber,
                CASE WHEN hitNumber = min_hitNumber 
                THEN ', i,'
                ELSE hitNumber_adjusted
                END as hitNumber_adjusted
                FROM t_hitNumber_adjusted
               ')
t_hitNumber_adjusted <- sqldf(query)

}
## FIN boucle


##
## CALCUL FINAL : utilisation de l'ordre simplifie calcule
##


## Jointure pour avoir hit minimum avec infos page -----------------------------------------------------------------
data_sunburst <- sqldf("
  SELECT a.*, b.page 
  FROM t_hitNumber_adjusted as a
  LEFT JOIN t_data as b 
  ON a.session_id = b.session_id AND a.hitNumber = b.hitNumber
")


##
## Boucle pour ajouter les colonnes de page pour chaque niveau
##


## DEBUT boucle -------------------------------------------------------------------------------------------
for (i in 1:nb_pages_max_pour_boucle){
query <- paste0('
                SELECT a.*, b.page as ', 'page_', i,'
                FROM data_sunburst as a
                LEFT JOIN (
                  SELECT session_id, page 
                  FROM data_sunburst 
                  WHERE hitNumber_adjusted = ', i,'
                  GROUP BY session_id
                ) as b
                ON a.session_id = b.session_id
              ')
data_sunburst <- sqldf(query)
}



## Structure de la table pour avoir l'info d'une session sur une seule ligne ---------------------------------------------
data_sunburst <- sqldf("
  SELECT *
  FROM data_sunburst
  WHERE hitNumber_adjusted = 1
")


## Ajout de l'info de FIN de sequence
## Construction de la requete avec une boucle -----------------------------------------------------------------------------
query <- 'SELECT page_1,'
for (i in 2:nb_pages_max_pour_boucle){
  query <- paste0( query,
                   'CASE WHEN ', 'page_', i-1, ' IS NULL = 0 AND ', 'page_', i, ' IS NULL = 1 THEN \'FIN\' ELSE ', 'page_', i, ' END as ', 'page_', i
                  )
  if(i != nb_pages_max_pour_boucle){
  query <- paste0(query, ',')  
  }    
}
query <- paste0( query,' FROM data_sunburst')
data_sunburst <- sqldf(query)


## Deselection des colonnes inutiles (conservation des colonnes page_1, page_2, etc) ------------------------------------------
help_query <- ''
for (i in 1:nb_pages_max_pour_boucle){
  help_query <- paste0(help_query, 'page_', i)
  if(i != nb_pages_max_pour_boucle){
    help_query <- paste0(help_query, ',')  
  } 
}
query <- paste0('SELECT ', help_query, ', count(page_1) as ', nom_colonne_size,'
  FROM data_sunburst
  GROUP BY ', help_query)
data_sunburst <- sqldf(query)

